const char digest_lua[] =
"-- box.digest.lua (internal file)\n"
"\n"
"do\n"
"\n"
"local ffi = require 'ffi'\n"
"\n"
"ffi.cdef[[\n"
"    /* from openssl/sha.h */\n"
"    unsigned char *SHA(const unsigned char *d, size_t n, unsigned char *md);\n"
"    unsigned char *SHA1(const unsigned char *d, size_t n, unsigned char *md);\n"
"    unsigned char *SHA224(const unsigned char *d, size_t n,unsigned char *md);\n"
"    unsigned char *SHA256(const unsigned char *d, size_t n,unsigned char *md);\n"
"    unsigned char *SHA384(const unsigned char *d, size_t n,unsigned char *md);\n"
"    unsigned char *SHA512(const unsigned char *d, size_t n,unsigned char *md);\n"
"    unsigned char *MD4(const unsigned char *d, size_t n, unsigned char *md);\n"
"\n"
"    /* from openssl/md5.h */\n"
"    unsigned char *MD5(const unsigned char *d, size_t n, unsigned char *md);\n"
"\n"
"    /* from libc */\n"
"    int snprintf(char *str, size_t size, const char *format, ...);\n"
"\n"
"\n"
"    typedef uint32_t (*crc32_func)(uint32_t crc,\n"
"        const unsigned char *buf, unsigned int len);\n"
"    extern crc32_func crc32_calc;\n"
"]]\n"
"\n"
"local ssl\n"
"if ssl == nil then\n"
"    pcall(function() ssl = ffi.load('ssl') end)\n"
"end\n"
"\n"
"\n"
"local def = {\n"
"    sha     = { 'SHA',    20 },\n"
"    sha1    = { 'SHA1',   20 },\n"
"    sha224  = { 'SHA224', 28 },\n"
"    sha256  = { 'SHA256', 32 },\n"
"    sha384  = { 'SHA384', 48 },\n"
"    sha512  = { 'SHA512', 64 },\n"
"    md5     = { 'MD5',    16 },\n"
"    md4     = { 'MD4',    16 }\n"
"}\n"
"\n"
"local m = {\n"
"    crc32 = function(str)\n"
"        if str == nil then\n"
"            str = ''\n"
"        else\n"
"            str = tostring(str)\n"
"        end\n"
"        return ffi.C.crc32_calc(4294967295, str, string.len(str))\n"
"    end,\n"
"\n"
"    crc32_update = function(crc, str)\n"
"        if str == nil then\n"
"            str = ''\n"
"        else\n"
"            str = tostring(str)\n"
"        end\n"
"        return ffi.C.crc32_calc(tonumber(crc), str, string.len(str))\n"
"    end\n"
"}\n"
"\n"
"if ssl ~= nil then\n"
"    local hexres = ffi.new('char[129]')\n"
"\n"
"    for pname, df in pairs(def) do\n"
"        local hfunction = df[1]\n"
"        local hsize = df[2]\n"
"\n"
"        m[ pname ] = function(str)\n"
"            if str == nil then\n"
"                str = ''\n"
"            else\n"
"                str = tostring(str)\n"
"            end\n"
"            local r = ssl[hfunction](str, string.len(str), nil)\n"
"            return ffi.string(r, hsize)\n"
"        end\n"
"        \n"
"        m[ pname .. '_hex' ] = function(str)\n"
"            if str == nil then\n"
"                str = ''\n"
"            else\n"
"                str = tostring(str)\n"
"            end\n"
"            local r = ssl[hfunction](str, string.len(str), nil)\n"
"            \n"
"            for i = 0, hsize - 1 do\n"
"                ffi.C.snprintf(hexres + i * 2, 3, \"%02x\",\n"
"                                ffi.cast('unsigned int', r[i]))\n"
"            end\n"
"            return ffi.string(hexres, hsize * 2)\n"
"        end\n"
"    end\n"
"else\n"
"    local function errorf()\n"
"        error(\"libSSL was not loaded\")\n"
"    end\n"
"    for pname, df in pairs(def) do\n"
"        m[ pname ] = errorf\n"
"        m[ pname .. '_hex' ] = errorf\n"
"    end\n"
"end\n"
"\n"
"\n"
"box.digest = m\n"
"\n"
"end\n"
""
;
